#include <system.h>
#include "lcd.h"
#include "serial.h"
#include "diskio.h"
#include <string.h>
#include "tff.h"

//###############################################################################
// using a 20 Mhz external crystal
#pragma DATA   _CONFIG1L, 00100100b  // USBPPL, CPU divide by 4 , PPL divide by 5                                                                                          
#pragma DATA   _CONFIG1H, 01001100b  // disable oscillator switchover,
// disable failsafe clock monitor, HSPPL                                                                                         
#pragma DATA   _CONFIG2L, 00111111b  // USB voltage regulator enabled,
// brownout set for 2.1 volts, hardware brownout only, PWRT disabled                                                                                       
#pragma DATA   _CONFIG2H, 00011110b  // watchdog timer disabled                                                                                      
#pragma DATA   _CONFIG3H, 10000001b  // MCLR enabled                                                                                          
#pragma DATA   _CONFIG4L, 10000001b  // DEBUG off, disable extended instructons, 
// LVP disabled, enable stack full/underflow reset                                                                                        
#pragma DATA   _CONFIG5L, 00001111b  // Read Code protection off                                                                                         
#pragma DATA   _CONFIG5H, 11000000b  // Read EEPROM and boot block protection off                                                                                          
#pragma DATA   _CONFIG6L, 00001111b  // Write Code protection off                                                                                          
#pragma DATA   _CONFIG6H, 11100000b  // Write EEPROM, boot block and
// config register protection off                                                                                                           
#pragma DATA   _CONFIG7L, 00001111b  // Table read protection off                                                                                           
#pragma DATA   _CONFIG7H, 01000000b  // Boot block table read protection off

// ################################################################################

//#define _SIN_WAVE_TEST_	//for test only
//#define _SOFT_SPI_			//Use SPI Emulation

//Set clock frequency
#pragma CLOCK_FREQ	20000000
#define ALBUM_MODE
/* I/O Name */
#define BHRCLK	0
#define BHWCLK	1
#define BHRE	2
#define STRD	3
#define	STWE	4

#define IOEPORT	portc
#define	IOEBIT	0
#define IOWPORT	portc
#define IOWBIT	1
#define IORPORT	portc
#define IORBIT	2
#define DATPORT portd
#define DATDIR trisd
#define ADRPORT	porte

#define ENABLE	1
#define DISABLE	0
#define IOCTRL(x) x ?  set_bit(IOEPORT,IOEBIT) : clear_bit(IOEPORT,IOEBIT)
#define IOW(x) x ?  clear_bit(IOWPORT,IOWBIT) : set_bit(IOWPORT,IOWBIT)

#define StartAD()	(adcon0=3)
#define ADBusy()	test_bit(adcon0,1)
#define GetAD()		(adresh & 0xfc)
#define ADTOLERANCE	5	//AD̋e덷

//Definition for SPI
#define XCSPORT		porta
#define XCSBIT		3
#define DCSPORT		porta
#define DCSBIT		4
#define SCI_READ	0x03
#define SCI_WRITE	0x02
#define DREQ_PORT	porta
#define DREQ_BIT	5
//for SPI Soft Emulation
#define SPI_CLK(x)	 (x) ? set_bit(portb,1) : clear_bit(portb,1)
#define SPI_MISO(x) (x) ? set_bit(portc,7) : clear_bit(portc,7)
#define SPI_MOSI() test_bit(portb,0)

//SCI Registers
#define	SCI_MODE	0x00
#define	SCI_STATUS	0x01
#define	SCI_BASS	0x02
#define	SCI_CLOCKF	0x03
#define	SCI_DECTIME	0x04
#define	SCI_AUDATA	0x05
#define	SCI_WRAM	0x06
#define	SCI_WRAMADR	0x07
#define	SCI_HDAT0	0x08
#define	SCI_HDAT1	0x09
#define	SCI_AIADDR	0x0a
#define	SCI_VOL		0x0b
#define	SCI_AICTRL0	0x0c
#define	SCI_AICTRL1	0x0d
#define	SCI_AICTRL2	0x0e
#define	SCI_AICTRL3	0x0f

//CLOCKF=12288000/2000+0x8000=0x9800	for Clock Doubling
#define	CLOCKF	0x9800
//==================================================================
//Definition for FAT System
#define READ 0
#define WRITE 1
#define ENABLE 1
#define DISABLE 0

#define CS_PORT porta
#define CS_DDR trisa
#define FLG_PORT portb
#define FLG_DDR trisb
#define EN_PORT portc
#define EN_DDR trisc
#define DATA_PORT portd
#define DATA_DDR trisd
#define ADR_PORT porte
#define ADR_DDR trise
#define SW_NEXT	1
#define	SW_PLAY	2
#define	SW_STOP	4
#define	SW_PAUSE	8
#define	SW_PREV		8

//tԂ̃tO
#define ST_STOP		0
#define ST_PLAY		1
#define ST_PAUSE	2

extern char read_ata(char adr);
extern void latch_high_byte(char dir);
extern void set_high_byte(char dir,char en);
unsigned char Disp_AD(void);

unsigned char SwitchStatus=0;		//݂̃XCb`̏
unsigned char PlayStatus=ST_STOP;	//݂̉t
//===============================================================================
//File Systems
//===============================================================================
FATFS fs;			//FATt@CVXe
BYTE _lbuf[256];	//t@CVXepobt@̉256oCg
BYTE _hbuf[256];	//t@CVXepobt@̏256oCg

//===============================================================================
//荞݃[`
//===============================================================================
static void OnTimer(void)
{
	disk_timerproc ();
}
//-------------------------------------------------------------------------------
void interrupt( void )
{
	//Handle RS232 Recieve interrupt
	if( pir1 & (1<< RCIF) )
	{
		clear_bit( pir1, RCIF ); //clear RS232 Recieve interrupt bit
	}
	//Handle timer0 interrupt
	if( intcon & (1<<T0IF) )
	{
		OnTimer(); //call timer 0 handler

		clear_bit( intcon, T0IF ); //clear timer 0 interrupt bit
	}
	//Handle timer1 interrupt
	if( pir1 & (1<<TMR1IF) )
	{
		clear_bit( pir1, TMR1IF ); //clear timer 1 interrupt bit
	}
	//Handle timer2 interrupt
	if( pir1 & (1<<TMR2IF) )
	{
		clear_bit( pir1, TMR2IF ); //clear timer 2 interrupt bit
	}
}

//===============================================================================
//SPI Functions
//===============================================================================
unsigned char SpiRW(unsigned char dat)
{
	unsigned char rdat;
#ifndef _SOFT_SPI_	
	sspbuf=dat;	//Send Data
	while(!test_bit(sspstat,0));
	rdat=sspbuf;
#else
	char i;
	rdat=0;
	for(i=0;i<8;i++){
		rdat<<=1;
		SPI_MISO(dat & 0x80);
		SPI_CLK(1);
		SPI_CLK(0);
		rdat|=SPI_MOSI();
		dat <<=1;
	}
#endif
	return rdat;
}
//---------------------------------------------------------------------------
void SciWrite(unsigned char cmd,unsigned short dat)
{
	unsigned char ldat,hdat;
	hdat=(dat >> 8);
	ldat=(dat & 0xff);
	while(!test_bit(DREQ_PORT,DREQ_BIT));	//fB[҂
	clear_bit(XCSPORT,XCSBIT);	//Enter Command Mode
	SpiRW(SCI_WRITE);
	SpiRW(cmd);
	SpiRW(hdat);
	SpiRW(ldat);
	set_bit(XCSPORT,XCSBIT);	//Exit Command Mode
	while(!test_bit(DREQ_PORT,DREQ_BIT));	//fB[҂
}
//---------------------------------------------------------------------------
unsigned short SciRead(unsigned char cmd)
{
	unsigned short dat;
	unsigned char ldat,hdat;

	while(!test_bit(DREQ_PORT,DREQ_BIT));	//fB[҂
	clear_bit(XCSPORT,XCSBIT);	//Enter Command Mode
	SpiRW(SCI_READ);
	SpiRW(cmd);
	hdat=SpiRW(0);
	ldat=SpiRW(0);
	set_bit(XCSPORT,XCSBIT);	//Exit Command Mode
	dat=(unsigned short)(hdat<<8)+ldat;
	while(!test_bit(DREQ_PORT,DREQ_BIT));	//fB[҂
	return dat;
}
//---------------------------------------------------------------------------
void SdiWrite(unsigned char *dat,unsigned char cnt)
{
	while(!test_bit(DREQ_PORT,DREQ_BIT));	//fB[҂
	clear_bit(DCSPORT,DCSBIT);	//Enter Data Mode
	while(cnt){
		SpiRW(*dat);
		dat++;
		cnt--;
	}
	set_bit(DCSPORT,DCSBIT);	//Exit Data Mode
}	
//---------------------------------------------------------------------------
void SdiSendData(unsigned char *dat,int cnt)
{
	//yf[^̑M
	//ő256oCg܂ňxɑM\
	
	unsigned char sndcnt;
	//Mf[^32oCgȓɕ
	while(cnt>0){
		if(cnt>32)
			sndcnt=32;
		else
			sndcnt=(unsigned char)cnt;
		SdiWrite(dat,sndcnt);
		dat+=sndcnt;
		cnt-=sndcnt;
	}
}
//---------------------------------------------------------------------------
void SpiInit()
{
	//SPȈ
#ifndef _SIN_WAVE_TEST_
	//ʏ탂[h======================================================
	set_bit(XCSPORT,XCSBIT);
#ifndef _SOFT_SPI_	
	sspstat=0xc0;	//SMP=1,CKE=1
	sspcon1=0x22;	//22h=SPI CLK=625KHz
#else
	sspcon1=0;	//SPI Port->I/O Port
	SPI_CLK(0);
#endif
	//Init VS1002d
	delay_ms(200);
	SciWrite(SCI_MODE,0x0804);	//Soft Reset
	delay_ms(500);
	SciWrite(SCI_MODE,0x0800);	//VS1002 Native mode
	delay_ms(10);	//wait least 11000 clock cycle
	//set CLOCKF
	SciWrite(SCI_CLOCKF,CLOCKF);
	delay_ms(10);	//wait least 11000 clock cycle
	sspcon1=0x21;	//21h=SPI CLK=2.5MHz(MAX CLK<4MHz)
	SciWrite(SCI_VOL,0);
#else
	//SinWave Test=====================================================
	const char testpat[8]={0x53,0xef,0x6e,0xc5,0,0,0,0};	//440Hz
	const char testend[8]={0x45,0x78,0x69,0x74,0,0,0,0};	

	set_bit(XCSPORT,XCSBIT);
#ifndef _SOFT_SPI_	
	sspstat=0xc0;	//SMP=1,CKE=1
	sspcon1=0x21;	//22h=SPI CLK=625KHz
#else
	sspcon1=0;	//SPI Port->I/O Port
#endif
	//Init VS1002d
	delay_ms(200);
	SciWrite(SCI_MODE,0x0804);	//Soft Reset
	delay_ms(500);
	SciWrite(SCI_MODE,0x0820);	//VS1002 Native mode with Test mode
	while(!test_bit(DREQ_PORT,DREQ_BIT));	//fB[҂
	//set CLOCKF
	SciWrite(SCI_CLOCKF,CLOCKF);
	SciWrite(SCI_AUDATA,0x2b11);	//Sampling rate=11.026KHz
	delay_ms(10);	//wait least 11000 clock cycle
	sspcon1=0x21;	//20h=SPI CLK=2.5MHz(MAX CLK<4MHz)
	SciWrite(SCI_VOL,0x2020);
	SdiSendData(testpat,8);
	while(1);
#endif
}
//===============================================================================
// n[hEFȀ
//===============================================================================
void InitHardware()
{
	//Port Setting
	porta=0x1e;
	lata = 0x1E;
	portb=0x80;
	portc=0x46;
	portd=0x00;
	porte=0x06;
	trisa = 0x21;
	trisb = 0x6d;
	trisc = 0x30;
	trisd = 0x00;
	trise = 0x00;
	adcon0 = 1;		//Enable A/D ch0
	adcon1 = 0x0e;	//RA0=Analog Input
	adcon2 = 0x3E;
	cmcon = 0x07;  
	//Set Timer0 mode
	clear_bit( t0con, T0CS ); //configure timer0 as a timer
	//Set prescaler assignment
	clear_bit( t0con, PSA ); //prescaler is assigned to timer0
	//Set prescaler rate
	set_bit( t0con, T0PS2 ); //prescaler rate 1:256
	set_bit( t0con, T0PS1 );
	set_bit( t0con, T0PS0 );
	//Set timer0 source edge selection
	set_bit( t0con, T0SE ); //increment on high-to-low transition on RA4/T0CKI pin
	intcon = 0xA0;	
	serialInit(26);			//VA|[g̏i̓n[hEFASCIpj
	SpiInit();				//SPȈ
	delay_ms(100);
	LcdInit(8);				//LCD̏
	LcdDisplayMode(1,0,0);		//Cursor OFF
}

//===============================================================================
//eI/O֐
//===============================================================================
void SelectIo(unsigned char adr)
{
	//AhXI
	if(adr & 0x80){
		IOCTRL(DISABLE);
	}else{
		ADRPORT=adr;
		IOCTRL(ENABLE);
	}
}
//------------------------------------------------------------------------------
unsigned char IoRead(unsigned char adr)
{
	//I/O̓ǂݍ
	unsigned char dat;
	SelectIo(adr);
	DATDIR=0xff;
	dat=DATPORT;
	DATDIR=0;
	SelectIo(-1);
	return dat;
}
//------------------------------------------------------------------------------
void IoWrite(unsigned char adr,unsigned char dat)
{
	//I/Ȍ
	DATDIR=0;
	DATPORT=dat;
	SelectIo(adr);
	IOW(ENABLE);
	IOW(DISABLE);
	SelectIo(-1);
}
//------------------------------------------------------------------------------
unsigned char ReadSwitch()
{
	//XCb`̓ǂݍ
	return IoRead(STRD) & 0x0f;
}
//------------------------------------------------------------------------------
unsigned char CheckSwitch()
{
	//{^ꂽǂ`FbN
	//ReadSwitch́Ã݂XCb`̏ԂԂA
	//CheckSwitch́A{^ꂽƂ̂݃rbg
	static unsigned char osw=0;
	unsigned char sw,dsw;
	sw=ReadSwitch();
	dsw=sw ^ osw;
	osw=sw;
	sw=dsw & sw;
	return sw;
}
//------------------------------------------------------------------------------
void SetLed(unsigned char led)
{
	//LED̓_iled̑Ήrbg_j
	IoWrite(STWE,led);
}
//------------------------------------------------------------------------------
void ShowAD(unsigned char val)
{
	//ADf[^̕\
	//LCDɉʂ\
	unsigned char wrk;
	char valstr[4];
	wrk=(val/100);
	val=val-(wrk*100);
	valstr[0]=wrk+'0';
	wrk=(val/10);
	val=val-(wrk*10);
	valstr[1]=wrk+'0';
	valstr[2]=val+'0';
	valstr[3]=0;
	LcdOutXy(0,1,"Vol=");
	LcdPuts(valstr);
}
//------------------------------------------------------------------------------
void SetVol(void)
{
	//ADǂݍ݁AʂZbg
	static unsigned char oadval=0;
	unsigned char adval,dif;
	unsigned short svol;

	//ADϊȂ牽Ȃ
	if(ADBusy())
		return;
	//AD̒lǂݍށi8rbĝ݁j
	adval=GetAD();
	if(adval>245)
		adval=255;
	//O̒lƂ̍߂
	if(adval>oadval)
		dif=adval-oadval;
	else
		dif=oadval-adval;
	//ADTOLERANCEȏ゠邩Aŏ/őlɂȂ̂݉ʂ𔽉f
	if((dif>ADTOLERANCE)||(adval==0)||(adval==0xff)){
		if(oadval!=adval){
#ifndef ALBUM_MODE
			ShowAD(adval);
#endif
			oadval=adval;
			adval=0xff-adval;
			if(adval==0xff)
				adval=0xfe;
			svol=(adval<<8)+adval;		
			SciWrite(SCI_VOL,svol);
		}
	}
	//ēxADϊJn
	StartAD();
}
//===============================================================================
//t@C̍Đ
//===============================================================================
void PlaySnd(char *fname,char *sfname)
{
	//w肳ꂽt@Ct
	//fname̓tpXAsfname̓t@Ĉ݁i\pj
	WORD rsize;
	char rbuf[128];	//obt@TCY255oCgȉŁA32̔{ɂĂ
	char lfn[128];		//t@C̎擾
	char lcdbuf[17];		//LCD\p̃obt@
	char led,len,y;
	signed char epos,dpos,lpos;
	char cnt,lcnt;
	/* File Objects */
     FIL fsrc;
     FRESULT res;

	lcdbuf[16]=0;
#ifdef ALBUM_MODE
	y=1;
#else
	y=0;
#endif

#ifdef _LFN_SUPPORT_
		len=f_GetLFN(fname,lfn,128);
		if(len){
			epos=strlen(lfn);
			dpos=-16;
			prints("PlayFile="); prints(lfn);	lf();
			LcdOutXy(0,y,"                 ");	//Clear Line
		}else{
			prints("PlayFile="); prints(sfname);	lf();
			LcdOutXy(0,y,"                 ");	//Clear Line
			LcdOutXy(0,y,sfname);
		}
#endif
	res = f_open(&fsrc,fname, FA_OPEN_EXISTING | FA_READ);
	if(res!=0){
		prints("File open error!\n");
		return;
	}

	cnt=0;
	led=8;
	lcnt=0;
	do{
		//LEDɍĐԂ\
		if(cnt>10){
			SetLed(led);
			led>>=1;
			if(!led)
				led=8;
			cnt=0;
		}else{
			cnt++;
		}
		if(lcnt>25){
			//Ȗ̃XN[
			lpos=dpos;
			for(cnt=0;cnt<16;cnt++){
				if((lpos>=0)&&(lpos<epos))
					lcdbuf[cnt]=lfn[lpos];
				else
					lcdbuf[cnt]=' ';
				lpos++;
			}
			LcdOutXy(0,y,lcdbuf);
			dpos++;
			if(dpos>=epos){
				dpos=-16;	
			}
			lcnt=0;
		}else{
			lcnt++;
		}
     	res = f_read(&fsrc, rbuf,128, &rsize);
 		if (res){
			prints("res=");printd(res);prints("\n\r");
			prints("Read Error!\n");
		}else{
			if(rsize){
				SdiSendData(rbuf,rsize);
				SetVol();	//ʂ̐ݒ
				SwitchStatus=CheckSwitch();
#ifdef ALBUM_MODE
				if(SwitchStatus==SW_PLAY){
					//Pauseꂽꍇ́AXCb`܂ő҂
					SetLed(0x0f);
					SwitchStatus=CheckSwitch();
					while(!SwitchStatus){
						SetVol();
						SwitchStatus=CheckSwitch();
					}
				}
				//NEXT܂STOPꂽꍇ̏
				if((SwitchStatus==SW_PREV)||(SwitchStatus==SW_NEXT)||(SwitchStatus==SW_STOP)){
					f_close(&fsrc);
					SetLed(0);
					if(SwitchStatus==SW_STOP)
						PlayStatus=ST_STOP;
					return;
				}
#else
				if(SwitchStatus==SW_PAUSE){	
					//Pauseꂽꍇ́AXCb`܂ő҂
					SwitchStatus=CheckSwitch();
					while(!SwitchStatus){
						SetVol();
						SwitchStatus=CheckSwitch();
					}
				}
				//NEXT܂STOPꂽꍇ̏
				if((SwitchStatus==SW_NEXT)||(SwitchStatus==SW_STOP)){
					f_close(&fsrc);
					SetLed(0);
					return;
				}
#endif
			}else{
				//prints("Read End\n");
				SetLed(0);
			}
		}

	}while((res==0)&&(rsize));
	f_close(&fsrc);
}
//------------------------------------------------------------------------------
#ifndef ALBUM_MODE
char ShowSubDir(char *path)
{
	//TutH_̌
	//ȏ㉺̃tH_͌Ȃ
    FILINFO finfo;
    struct _DIR dirs;
    char fname[32];
	if (f_opendir(&dirs, path)!="") {
     	while((f_readdir(&dirs, &finfo) == FR_OK) && finfo.fname[0]) {
			//prints(finfo.fname);
       		if (finfo.fattrib & AM_DIR) {
				//tH_̏ꍇ͉Ȃ
           	} else {
				strcpy(fname,path);
				strcat(fname,"/");
				strcat(fname,finfo.fname);
				PlaySnd(fname,finfo.fname);	//t@C̏ꍇ́ApXǉčĐ
				if(SwitchStatus==SW_STOP)
					return 0;
            }
        }
    }else{
		return 0;
	}
	return 1;
}
//------------------------------------------------------------------------------
char ShowDir()
{
	//tH_̌
    FILINFO finfo;
    struct _DIR dirs;
	char newpath[20];
	char album[128];
	char len;
	const char *path="";

	newpath[0]='/';
	if (f_opendir(&dirs, path)!="") {
     	while((f_readdir(&dirs, &finfo) == FR_OK) && finfo.fname[0]) {
        	if (finfo.fattrib & AM_DIR) {
				strcpy(&newpath[1],finfo.fname);
				//Ao̎擾
				len=f_GetLFN(newpath,album,128);
              	prints("Album=");	
				if(len){
					prints(album);
				}else{
					prints(finfo.fname);
				}
				lf();
				//tH_̏ꍇ́AɌ
				ShowSubDir(newpath);	//X^bNI[o[t[h߁AJ[W͎gȂ
				if(SwitchStatus==SW_STOP)
					return 0;
            } else {
				PlaySnd(finfo.fname,finfo.fname);	//t@C̏ꍇ́AĐJn
				if(SwitchStatus==SW_STOP)
					return 0;
            }
        }
    }else{
		return 0;
	}
	return 1;
}
#else
//===============================================================================
// Ao[h
//===============================================================================
char SelectMusic(char *path)
{
	//TutH_̌
	//ȏ㉺̃tH_͌Ȃ
    FILINFO finfo;
    struct _DIR dirs;
    char fname[32];
	char lfn[128];
	char len;
	char retc;
	const char *path="";
	
	retc=0;	
	if (f_opendir(&dirs, path)!="") {
		//ŏ̋Ȃ̎擾
		if((f_readdir(&dirs, &finfo) == FR_OK) && finfo.fname[0]) {
			if(!(finfo.fattrib & AM_DIR)){
				retc=1;
			}else{
				//Ȃ̌
				while((f_next_dir(&dirs, &finfo) == FR_OK) && finfo.fname[0]) {
					if(!(finfo.fattrib & AM_DIR)){
						retc=1;
						break;
					}
				}
			}
		}
    }
	if(!retc){
		LcdOutXy(0,1,"No music data.  ");
		prints("Sub directory open error!\n");
		return 0;
	}

	while((PlayStatus==ST_PLAY)||(PlayStatus==ST_PAUSE)){
		strcpy(fname,path);
		strcat(fname,"/");
		strcat(fname,finfo.fname);
		if((PlayStatus==ST_PAUSE)&&(retc)){		
			//Ȗ̕\
			LcdOutXy(0,1,"                ");
			//Ao̎擾
			len=f_GetLFN(fname,lfn,128);
			prints("Music=");	
			if(len){
				prints(lfn);
				lfn[16]=0;
				LcdOutXy(0,1,lfn);
			}else{
				LcdOutXy(0,1,finfo.fname);
				prints(finfo.fname);
			}
			lf();
		}else{
			PlaySnd(fname,finfo.fname);	//t@C̏ꍇ́ApXǉčĐ
		}

		//XCb`̎擾
		if(PlayStatus==ST_PLAY){
			SwitchStatus=SW_NEXT;
		}else{
			do{
				SwitchStatus=CheckSwitch();
			}while((SwitchStatus!=SW_STOP)&&(SwitchStatus!=SW_NEXT)&&(SwitchStatus!=SW_PLAY)&&(SwitchStatus!=SW_PREV));
			delay_ms(30);
		}
		switch(SwitchStatus){
		case SW_NEXT:
			//̋
			retc=0;
			while((f_next_dir(&dirs, &finfo) == FR_OK) && finfo.fname[0] ){
				if(!(finfo.fattrib & AM_DIR)){
					retc=1;
					break;
				}
			}
			if((!retc)&&(PlayStatus==ST_PLAY))
				return 1;	
			break;
		case SW_PREV:
			//Ő
			retc=0;
			while((f_prev_dir(&dirs, &finfo) == FR_OK) && finfo.fname[0] ){
				if(!(finfo.fattrib & AM_DIR)){
					retc=1;
					break;
				}
			}
			break;
		case SW_PLAY:
			//Ȗڂ̑I
			if(PlayStatus==ST_PAUSE)
				PlayStatus=ST_PLAY;
			else if(PlayStatus==ST_PLAY)
				PlayStatus=ST_PAUSE;
			break;
		case SW_STOP:
			PlayStatus=ST_STOP;
			break;
		}
	}
}

char SelectAlbum()
{
	//gbvtH_̌
    FILINFO finfo;
    struct _DIR dirs;
	char newpath[20];
	char album[128];
	char len;
	char retc;
	const char *path="";
	
	newpath[0]='/';
	PlayStatus=ST_STOP;		//t~
	retc=0;	
	if (f_opendir(&dirs, path)!="") {
		if((f_next_dir(&dirs, &finfo) == FR_OK) && finfo.fname[0]) {
			if (finfo.fattrib & AM_DIR) {
				retc=1;
			}else{
				while((f_next_dir(&dirs, &finfo) == FR_OK) && finfo.fname[0]) {
					if (finfo.fattrib & AM_DIR) {
						retc=1;
						break;
					}
				}
			}
		}
    }
	if(!retc){
		LcdOutXy(0,0,"No album data.  ");
		prints("Directory open error!\n");
		return 0;
	}
	while(1){
		//Ao̕\
		if(retc){
			LcdOutXy(0,0,"                ");
			strcpy(&newpath[1],finfo.fname);
			//Ao̎擾
			len=f_GetLFN(newpath,album,128);
			prints("Album=");	
			if(len){
				prints(album);
				album[16]=0;
				LcdOutXy(0,0,album);
			}else{
				LcdOutXy(0,0,finfo.fname);
				prints(finfo.fname);
			}
			lf();
		}
		retc=0;
		if((PlayStatus==ST_PLAY)||(PlayStatus==ST_PAUSE)){
			//t́A̋ȂI
			SelectMusic(newpath);
		}

		//XCb`̎擾
		if(PlayStatus==ST_PLAY){
			SwitchStatus=SW_NEXT;
		}else{
			do{
				SwitchStatus=CheckSwitch();
			}while((SwitchStatus!=SW_NEXT)&&(SwitchStatus!=SW_PLAY)&&(SwitchStatus!=SW_PREV));
			delay_ms(30);	//`^Oh~
		}
		switch(SwitchStatus){
		case SW_NEXT:
			//̃Ao
			retc=0;
			while((f_next_dir(&dirs, &finfo) == FR_OK) && finfo.fname[0] ){
				if(finfo.fattrib & AM_DIR){
					retc=1;
					break;
				}
			}
			if(!retc)
				PlayStatus=ST_STOP;
			break;
		case SW_PREV:
			//ÕAo
			retc=0;
			while((f_prev_dir(&dirs, &finfo) == FR_OK) && finfo.fname[0] ){
			//while((f_prev_readdir(&dirs, &finfo) == FR_OK) && finfo.fname[0] ){
				if(finfo.fattrib & AM_DIR){
					retc=1;
					break;
				}
			}
			break;
		case SW_PLAY:
			//Ȗڂ̑I
			PlayStatus=ST_PAUSE;
			break;
		}
	}
}
#endif

//===============================================================================
//C֐
//===============================================================================
void main( void )
{
	FatFs=&fs;		//t@CVXȅ
	InitHardware();

	memset((void *)FatFs,0,sizeof(FATFS));	//t@CVXȅ
	FatFs->win.lbuff = _lbuf;	//obt@̃Zbg
	FatFs->win.hbuff = _hbuf;
	memset(FatFs->win,0,512);	//obt@ONA

	//ʂ̕\
	LcdPuts("PIC18F4550Ready.");
	prints("PIC18F4550 Test...\n");
	//fBXN̏
	if(disk_initialize() & 0x01){ 
		//s
		prints("disk is not initialized\n");
	}else{
#ifdef ALBUM_MODE
		SelectAlbum();
#else
		while(1){
			while(CheckSwitch()!=SW_PLAY);	//PLAY܂ő҂
			ShowDir();	//tH_ĉtJn
		}
#endif
	}
	//Endless loop
	while( 1 );

}
